﻿/// ------------------------
/// #VARIABLES
/// ------------------------

// Specified whether any calls are in progress. @type bool
var calling = false;

// A pending call which the web client has received. @type Call
var pendingCall = null;

// The caller or the callee party of these sessions are one of the displayed users. @type {Object.<string,Session>}
var sessionsParticipating = {};

// Defines user statuses.
var UserStatus = {
    AVAILABLE: "availableStatus",
    INCALL: "incallStatus",
    DISABLED: 'offlineStatus',
    OFFLINE: "offlineStatus",
};

/// ------------------------
/// #INIT
/// ------------------------

/**
* This method will be called when the page loaded.
* @param address {string} - The address of the PBX.
* @param securityToken {string} - The security token for the client.
*/
function init(address, user, password) {

    // check if Ozeki Phone System is available
    if (typeof (OzWebClient) == "undefined") {
        $("#userStatusString").html("Server offline");
        return;
    }

    // subscribe to the events of the webclient
    OzWebClient.onConnectionStateChanged(updateCurrentUserData);
    OzWebClient.onSessionCreated(sessionCreated);
    OzWebClient.onSessionClosed(sessionClosed);
    OzWebClient.onUserInfoReceived(userInfoReceived);
    OzWebClient.onUserRemoved(removeUser);
    OzWebClient.onIncomingCall(incomingCall);
    OzWebClient.onCallHistoryInfoReceived(callHistoryInfoReceived);
    OzWebClient.onMessageReceived(messageReceived);

    // update labels on the GUI
    $("#userStatusString").html("Connecting...");
    $("#currentUserName").html(user);

    // connect to Ozeki Phone System
    OzWebClient.connect(address, user, { Password: password });
}

/// ------------------------
/// #HISTORY
/// ------------------------

/**
* Call history infos received from Ozeki Phone System XE.
* @param {Object.<Object>} An Object containing call historty infos.
*/
function callHistoryInfoReceived(infos) {
    log('call history info received');

    for (var j = 0; j < infos.CallHistoryEntries.length; j++) {

        var i = infos.CallHistoryEntries[j];
        var sessionInfo = {
            caller: i.Source,
            callee: i.Destination,
            sessionId: i.SessionId,
            sessionState: i.CallState,
            startTime: i.StartTime,
        };

        //Check the users who participated in the specified call

        var user = OzWebClient.helper.getUserByExtension(i.Source);
        if (user != null) {
            addCallHistoryEntry(user, sessionInfo);
        }

        user = OzWebClient.helper.getUserByExtension(i.Destination);
        if (user != null) {
            addCallHistoryEntry(user, sessionInfo);
        }
    }
}

/// ------------------------
/// #USER
/// ------------------------

/**
* Updates the data of the signed in user on the GUI. 
* @param user {User} - The information about the current user.
*/
function updateCurrentUserData(userInfo) {

    if (userInfo.State == ConnectionState.ACCESS_GRANTED) {
        $("#userStatusString").html("Available");
        $("#userStatusOffline").hide();
        $("#userStatusOnline").show();
    }
    else {
        $("#userStatusString").html("Offline");
        $("#userStatusOffline").show();
        $("#userStatusOnline").hide();
    }
}

/**
* Adds a user to the list on the GUI.
* @param user {User} - The User class instance.
*/
function addUser(user) {

    // update the extensions of the user
    for (var i = 0; i < user.devices.length; i++) {
        var ext = OzWebClient.helper.getExtensionById(user.devices[i]);
        if (ext == null)
            continue;

        ext.onExtensionStatusChanged(extensionStatusChanged);
    }

    // append user info to GUI
    var div = '' +
        '<div class="userDetails" id="user_' + user.username + '">' +
            '<span class="userFullName" id="userFullName_' + user.username + '">' +
                user.fullName +
            '</span>' +
            '<span class="userExtensionNumber" id="userDeviceNumber_' + user.username + '"></span>' +
            '<div class="userCalls">'+
                '<a id="userNumberOfCalls_' + user.username + '" href="javascript:showSessionDetails(\'' + user.username + '\')"></a>' +
            '</div>' +
            '<div class="userActions">' +
                '<div class="left hoverCursor" id="callUserPhone_' + user.username + '"></div>' +
                '<div class="left hoverCursor" id="callUserMobile_' + user.username + '"></div>' +
                '<div class="left hoverCursor" id="sendUserEmail_' + user.username + '"></div>' +
                '<div class="left hoverCursor">' +
                    '<a href="javascript:openChat(\''+ user.username + '\', \'' + user.fullName + '\')"><img src="assets/chat_icon.png" class="icon" title="Send an instant message"></a>' +
                '</div>' +
                 '<div class="right hoverCursor">' +
                    '<a href="javascript:showSessionDetails(\'' + user.username + '\')"><img src="assets/call_history.png" class="icon" title="View call history"></a>' +
                '</div>' +
            '</div>' +
        '</div>';

    $("#userContainer").append(div);


    // append call details base panel
    var callDetailsDiv = '' +
        '<div id="userSessionDetails_' + user.username + '" class="callDetailsPanel panel hidden">' +
            '<div class="windowLabel">'+
                '<div class="left">Call history: ' + user.fullName + '</div>'+
                '<div class="right">'+
                    '<a href="javascript:void()" onclick="closeSessionDetailsPanel(\'' + user.username + '\')" title="Close panel"><img src="assets/close_icon.png" class="icon" /></a>' +
                '</div>'+
                '<div class="clear"></div>' +
            '</div>' +
            '<div class="callDetailsHeader">' +
                '<div class="callDetailsDate left">Date</div>' +
                '<div class="callDetailsCallParty left">Caller</div>' +
                '<div class="callDetailsCallParty left">Callee</div>' +
                '<div>State</div>' +
            '</div>' +
            '<div id="firstSessionEntry_' + user.username + '"></div>' +
        '</div>';

    $("#callDetailsContainer").append(callDetailsDiv);


    // update the status of the user on the GUI
    setUserStatus(user);

    // update the call status of the user on the GUI
    updateUserCallStatus(user);

    // update the user data on the GUI
    updateUserData(user);

    // Initiate a get call history request towards Ozeki Phone System XE
    var deviceNumber = user.devices[0];
    if (deviceNumber == undefined)
        return;

    OzWebClient.getCallHistory({
        pageNumber: 0,
        pageRows: 10,
        general: deviceNumber
    });
}

/**
* Removes a user from the GUI.
* @param {string} username - The username of the user.
*/
function removeUser(username) {
    $('#user_' + username).empty().remove();
}

/**
* Refreshes the attributes of the specified user.
* @param {User} user - A User class instance.
*/
function updateUserData(user) {

    log('user info changed: ' + user.username);

    var div = $('#user_' + user.username);

    // update full name on the GUI
    var userNameDiv = div.find('#userFullName_' + user.username);
    if (userNameDiv.length != 0) {
        userNameDiv.html(user.fullName);
    }

    // update the phone number of the user on the GUI
    var callUserPhoneDiv = div.find('#callUserPhone_' + user.username);
    if (callUserPhoneDiv.length != 0) {
        var deviceNumber = user.devices[0];
        if (deviceNumber == undefined)
            deviceNumber = '-';

        callUserPhoneDiv.html('<img src="assets/answer_icon.png" class="icon" title="Call user\'s phone (' + deviceNumber + ')" onclick="call(\'' + deviceNumber + '\')">');
        $("#userDeviceNumber_" + user.username).html("(" + deviceNumber + ")");
    }

    // update the mobile number of the user on the GUI
    var callUserMobileDiv = div.find('#callUserMobile_' + user.username);
    if (callUserMobileDiv.length != 0) {
        var mobileNumber = user.mobileNumber;
        if (mobileNumber == undefined || mobileNumber == "")
            mobileNumber = 'unavailable';

        callUserMobileDiv.html('<img src="assets/call_mobile_icon.png" class="icon" title="Call user\'s mobile (' + mobileNumber + ')" onclick="call(\'' + mobileNumber + '\')">');
    }

    // update the email address of the user on the GUI
    var sendUserEmailDiv = div.find('#sendUserEmail_' + user.username);
    if (sendUserEmailDiv.length != 0) {
        var emailAddress = user.emailAddress;
        if (emailAddress == undefined || emailAddress == "")
            emailAddress = 'unavailable';

        sendUserEmailDiv.html('<a href="mailto:' + user.emailAddress + ' "><img src="assets/clickToEmail_icon.png" class="icon" title="Send an email to user ('+ emailAddress +')"></a>');
    }

    // refresh the status of the user
    setUserStatus(user);
}

/**
* Updates the background of the user on the GUI.
* @param {User} user - The specified user class instance.
* @param {UserStatus} status - The status of the user.
*/
function updateUserStatus(user, status) {
    $('#user_' + user.username).removeClass();
    $('#user_' + user.username).addClass("userDetails");
    $('#user_' + user.username).addClass(status);
}

/**
* Updates the call count display of a user on the GUI.
* @param {User} user - The specified user class instance.
*/
function updateUserCallStatus(user) {
    var callsOfUser = OzWebClient.helper.getActiveCallsOfUser(user.username);
    var numberOfCalls = count(callsOfUser);
    if (numberOfCalls > 0) {
        $('#userNumberOfCalls_' + user.username).html(numberOfCalls + " call(s)");
    } else
        $('#userNumberOfCalls_' + user.username).html("");
}

/**
* Updates the user status on the GUI based on its availability, extensions and calls.
* @param {User} user - The specified user class instance.
*/
function setUserStatus(user) {

    // user is disabled
    if (!user.enabled) {
        updateUserStatus(user, UserStatus.DISABLED);
        return;
    }

    // user has no extensions
    if (user.devices.length == 0) {
        updateUserStatus(user, UserStatus.OFFLINE);
        return;
    }

    // if user has an at least one active call, update theri status and print the number of calls
    var calls = OzWebClient.helper.getActiveCallsOfUser(user);
    if (count(calls) > 0) {
        updateUserStatus(user, UserStatus.INCALL);
        updateUserCallStatus(user);
        return;
    }

    // if one of associated extension is registered, the user is online
    for (var i = 0; i < user.devices.length; i++) {
        var device = user.devices[i];
        var extension = OzWebClient.helper.getExtensionById(device);

        if (extension != null && (extension.extensionStatus == "Enabled" || extension.extensionStatus == 'Online')) {
            updateUserStatus(user, UserStatus.AVAILABLE);
            return;
        }
    }

    //Refresh the status of the user
    updateUserStatus(user, UserStatus.OFFLINE);
}

/**
* Appends a session entry to the first row of the call history.
* @param {User} user - The specified user class instance.
* @param {Object} sessionInfo - An object containing session information.
*/
function addActiveSessionDetails(user, sessionInfo) {

    var id = '#sessionDetails_' + sessionInfo.sessionId + '_' + user.username;
    if ($(id).length != 0) {
        return;
    }

    var detailsHTML = '' +
        '<div id="sessionDetails_' + sessionInfo.sessionId + '_' + user.username + '" class="callDetailsBody">' +
            '<div class="callDetailsDate left">' + sessionInfo.startTime + '</div>' +
            '<div class="callDetailsCallParty left">' + sessionInfo.caller + '</div>' +
            '<div class="callDetailsCallParty left">' + sessionInfo.callee + '</div>' +
            '<div id="detailsSessionState_' + sessionInfo.sessionId + '_' + user.username + '" class="callDetailsCallState left">' + sessionInfo.sessionState + '</div>' +
            '<div id="sessionActions_' + sessionInfo.sessionId + '_' + user.username + '" class="callDetailsActions right">' +
                '<div class="left">' +
                    '<a href="javascript:hangupSession(\'' + sessionInfo.sessionId + '\')" title="Hang up"><img src="assets/hangup_icon.png" class="icon" /></a>' +
                '</div>' +
                '<div id="holdCall_' + sessionInfo.sessionId + '_' + user.username + '" class="left hidden">' +
                    '<a href="javascript:holdSession(\'' + sessionInfo.sessionId + '\')" title="Put the call on call"><img src="assets/hold_icon.png" class="icon" /></a>' +
                '</div>' +
                '<div id="unholdCall_' + sessionInfo.sessionId + '_' + user.username + '" class="left hidden">' +
                    '<a href="javascript:unholdSession(\'' + sessionInfo.sessionId + '\')" title="Take the call off hold"><img src="assets/unhold_icon.png" class="icon" /></a>' +
                '</div>' +
                '<div class="clear"></div>' +
            '</div>' +
            '<div class="clear"></div>' +
        '</div>';

    $("#firstSessionEntry_" + user.username).after(detailsHTML);
}


/**
* Appends a call history entry to the call history of the user.
* @param {User} user - The specified user class instance.
* @param {Object} sessionInfo - An object containing session information.
*/
function addCallHistoryEntry(user, sessionInfo) {

    var id = '#sessionDetails_' + sessionInfo.sessionId + '_' + user.username;
    if ($(id).length != 0) {
        return;
    }

    var detailsHTML = '' +
    '<div id="sessionDetails_' + sessionInfo.sessionId + '_' + user.username + '" class="callDetailsBody">' +
        '<div class="callDetailsDate left">' + sessionInfo.startTime + '</div>' +
        '<div class="callDetailsCallParty left">' + sessionInfo.caller + '</div>' +
        '<div class="callDetailsCallParty left">' + sessionInfo.callee + '</div>' +
        '<div id="detailsSessionState_' + sessionInfo.sessionId + '_' + user.username + '" class="callDetailsCallState left">' + sessionInfo.sessionState + '</div>' +
        '<div class="clear"></div>' +
    '</div>';

    $('#userSessionDetails_' + user.username).append(detailsHTML);
}

/**
* Opens the details of the session of the specified user.
* @param {string} username - The username of the user.
*/
function showSessionDetails(username) {
    $(".callDetailsPanel").hide();
    $("#userSessionDetails_" + username).show();
}

/**
* Closes the call details panel of the specified user.
* @param {string} username - The username of the user.
*/
function closeSessionDetailsPanel(username) {
    $("#userSessionDetails_" + username).hide();
}

/**
* JS API callback
* A User has been added or modified in Ozeki Phone System XE. After a successfull connection, all user infos will be received through this event.
* @param {string} username - The username of the user.
* @param {User} user - The corresponding User class instance.
*/
function userInfoReceived(username, user) {
    log('User info received: ' + user.username);
    log(user);

    // if the user exists on the GUI, updates their status, otherwise inserts a new user to the list.
    var userExists = $('#user_' + username).length != 0;
    if (userExists) {
        updateUserData(user);
        return;
    }
    else {
        addUser(user);
    }
}

/**
* JS API callback
* Update the corresponding user status when an extension status has changed.
*/
function extensionStatusChanged(extension, status) {
    log('extension status changed: ' + extension.extensionId);
    log(extension);

    var user = OzWebClient.helper.getUserByExtension(extension.extensionId);
    if (user == null)
        return;

    setUserStatus(user);
}

/// ------------------------
/// #CALLS
/// ------------------------

/**
* JS API callback
* The web client gets an incoming call.
* @param {Call} call - The created Call class instance.
*/
function incomingCall(call) {
    log('incoming call');

    if (pendingCall) {
        log('incoming call rejected');
        call.reject();
    }

    pendingCall = call;
    pendingCall.onCallStateChanged(callStateChanged);

    // update GUI elements
    $('#answerBtn').show();
    $('#rejectBtn').show();
    $('#calleeDisplay').html(getUserFromDialedNumber(call.getOtherParty()));
}

/**
* Accepts the pending call.
*/
function acceptCall() {
    if (pendingCall == null)
        return;

    pendingCall.accept();
}


/**
* Rejects the incoming call.
*/
function rejectCall() {
    if (pendingCall == null)
        return;

    pendingCall.reject();
}


/**
* Hangs up the current call of the user.
*/
function hangupCall() {
    if (pendingCall == null)
        return;

    pendingCall.hangUp();
}

/**
* Holds the current call of the user.
*/
function holdCall() {
    if (pendingCall == null)
        return;

    pendingCall.hold();
}

/**
* Unholds the current call of the user.
*/
function unholdCall() {
    if (pendingCall == null)
        return;

    pendingCall.unHold();
}

/**
* Setup a call with the specified target extension.
* @param {string} dialNumber - The dialed number.
*/
function call(dialNumber) {

    // validate parameters
    if (!dialNumber || typeof(dialNumber) == undefined || dialNumber == 'unavailable')
        return;

    if (calling) {
        log('Call is already in progress');
        return;
    }

    if (pendingCall) {
        log('Call exists, aborting new call');
        return;
    }

    // create and start the call on the PBX
    var call = OzWebClient.createCall(dialNumber);
    call.onCallStateChanged(callStateChanged);
    call.start();

    calling = true;
    pendingCall = call;
}

/**
* Sets the current state of the active call.
* @param {string} - The current state of the active call.
*/
function callStateChanged(call, callState) {

    // update the parameters of the call on the GUI
    var callStateFormatted = callState.toLowerCase().replace('_', ' ');
    $('#callState').html(callStateFormatted);
    $('#calleeDisplay').html(getUserFromDialedNumber(call.getOtherParty()));

    // if the call has ended, clear the display after 3 seconds.
    if (callState == CallState.COMPLETED || callState == CallState.NOT_FOUND || callState == CallState.REJECTED || callState == CallState.BUSY) {
        setTimeout(function () { $('#callState').empty(); $("#calleeDisplay").empty(); }, 3000);
        pendingCall = null;
        calling = false;
        $('#hangupBtn').hide();
        $('#answerBtn').hide();
        $('#rejectBtn').hide();
        $('#holdBtn').hide();
        $('#unholdBtn').hide();
        return;
    }

    // update the visibility of the buttons on the GUI
    if (callState == CallState.RINGING) {
        $('#answerBtn').show();
        if (call.isIncoming) {
            $('#rejectBtn').show();
        } else {
            $('#hangupBtn').show();
        }
    }
    else if (callState == CallState.IN_CALL) {
        $('#hangupBtn').show();
        $('#answerBtn').hide();
        $('#rejectBtn').hide();
        $('#holdBtn').show();
        $('#unholdBtn').hide();
    }
    else if (callState == CallState.HOLD) {
        $('#holdBtn').hide();
        $('#unholdBtn').show();

    } else if (callState == CallState.RINGING) {
        $('#hangupBtn').show();
        $('#answerBtn').show();
        $('#rejectBtn').hide();
        $('#holdBtn').hide();
        $('#unholdBtn').hide();
    }
}

/**
* Gets the corresponding user from the extension ID.
* @param {string} dialedNumber - The dialed number.
*/
function getUserFromDialedNumber(dialedNumber) {

    var user = OzWebClient.helper.getUserByExtension(dialedNumber);
    if (user == null) {
        return dialedNumber;
    }

    return user.fullName;
}





/// ------------------------
/// #MESSAGE
/// ------------------------

/**
* Creates a new chat window or shows an existing chat window with the specified user.
* @param {string} username - The username of the user.
* @param {string} fullName - The full name of the user.
*/
function openChat(username, fullName) {

    if ($("#tabItem_" + username).length == 0) {
        newChatTab(username, fullName);
        $("#chatPanel").show();
        return;
    }

    showChatTab(username);
}


/**
* Sends a message to the given user.
* @param {string} userId - The ID of the user who will receive the message.
* @param {string} content - The content of the message.
*/
function sendMessage(userId, content) {

    // validate params
    if (!userId)
        return;

    if (!content)
        return;

    // append the sent mesasge to the chat history
    var time = getCurrentTime();
    appendMessage(userId, content, time, true, webphoneExtensionNumber);
    
    // send message to the Phone System
    var user = OzWebClient.helper.getUserById(userId);
    if (!user)
        return;

    for (i = 0; i < user.devices.length; i++) {
        OzWebClient.sendMessage(user.devices[i], content);
    }
    
}

/**
* Function to handle onMessageReceived events (the web client has received a message from Ozeki Phone System XE).
* @param {string} source - The ID of the extension who sent the message.
* @param {string} sender - The sender of the message.
* @param {string} recipient - The recipient of the message.
* @param {string} content - The content of the message.
*/
function messageReceived(source, sender, recipient, content) {
    log('received message from ' + source);
    
    // get the user for the extension ID (source)
    var user = OzWebClient.helper.getUserByExtension(source);
    if (user == null)
        return;

    // open a new chat tab if necessary
    newChatTab(user.username, user.fullName);

    // append the received mesasge to the chat history
    var time = getCurrentTime();
    appendMessage(user.username, content, time, false, user.fullName);

    // update the style of the tab item
    if (currentChatUser != user.username)
        $("#tabItem_" + user.username).addClass("highlightTab");

    $("#chatPanel").show();
}




/// ------------------------
/// #SESSION
/// ------------------------

/**
* JS API callback
* A session has been created in Ozeki Phone System XE.
* @param {Session} sessionInfo - The corresponding Session class instance.
*/
function sessionCreated(sessionInfo) {

    // update the user details on the GUI (background color, calls etc.)
    var affectedUsers = OzWebClient.helper.getUsersBySession(sessionInfo);
    for (var i in affectedUsers) {
        var current = affectedUsers[i];
        updateUserStatus(current, UserStatus.INCALL);
        updateUserCallStatus(current);
        addActiveSessionDetails(current, sessionInfo);
    }

    // add session to the local cache
    sessionInfo.onSessionStateChanged(sessionStateChanged);
    sessionsParticipating[sessionInfo.sessionId] = sessionInfo;
};


/**
* JS API callback
* The state of one of the sessions has changed.
* @param {Session} session - The session object.
* @param {string} sessionState - The new state of the session.
*/
function sessionStateChanged(session, sessionState) {
    var sessionParticipating = sessionsParticipating[session.sessionId];
    if (!sessionParticipating)
        return;

    // update the user details on the GUI (background color, calls etc.)
    var affectedUsers = OzWebClient.helper.getUsersBySession(sessionParticipating);
    for (var i in affectedUsers) {
        var current = affectedUsers[i];

        // change session status
        $("#detailsSessionState_" + session.sessionId + '_' + current.username).html(session.sessionState);

        if (sessionState == SessionState.ONHOLD) {
            $("div[id^='holdCall_" + session.sessionId + "']").hide();
            $("div[id^='unholdCall_" + session.sessionId + "']").show();
        }

        if (sessionState == SessionState.INCALL) {
            $("div[id^='holdCall_" + session.sessionId + "']").show();
            $("div[id^='unholdCall_" + session.sessionId + "']").hide();
        }
    }
}

/**
* JS API callback
* One of the sessions in Ozeki Phone System XE has ended.
* @param {string} sessionId - The ID of the terminated session.
*/
function sessionClosed(sessionId) {
    var sessionParticipating = sessionsParticipating[sessionId];
    if (!sessionParticipating)
        return;

    // update the user details on the GUI (background color, calls etc.)
    var affectedUsers = OzWebClient.helper.getUsersBySession(sessionParticipating);    
    for (var i in affectedUsers) {
        var currentUser = affectedUsers[i];

        //remove control buttons
        $('#sessionActions_' + sessionId + '_' + currentUser.username).remove();

        setUserStatus(currentUser);
        updateUserCallStatus(currentUser);
    }

    // remove session from the local cache
    delete sessionsParticipating[sessionId];
}


/**
* Hangs up the session with the specified session ID.
* @param {string} sessionId - The specified session ID.
*/
function hangupSession(sessionId) {
    var session = OzWebClient.activeSessions[sessionId];
    if (!session)
        return;

    session.hangup();
}

/**
* Puts the session with the specified session ID on hold.
* @param {string} sessionId - The specified session ID.
*/
function holdSession(sessionId) {
    var session = OzWebClient.activeSessions[sessionId];
    if (!session)
        return;

    session.hold();
}

/**
* Takes the session with the specified session ID off from hold.
* @param {string} sessionId - The specified session ID.
*/
function unholdSession(sessionId) {
    var session = OzWebClient.activeSessions[sessionId];
    if (!session)
        return;

    session.unhold();
}


/// ------------------------
/// #UTIL
/// ------------------------

/**
* Count the properties of an object
* @param {Object} obj - The specified object.
*/
function count(obj) {
    var count = 0;
    for (var prop in obj) {
        count++;
    }

    return count;
}

/**
* Append a log message to the console of the browser.
* @param {string} msg - The log message.
*/
function log(msg) {
    console.log(msg);
}


/**
* Gets the current time format for a chat entry.
*/
function getCurrentTime() {
    var currentTime = new Date();
    var hours = currentTime.getHours();
    var minutes = currentTime.getMinutes();

    if (minutes < 10)
        minutes = "0" + minutes;

    var time = hours + ":" + minutes;

    return time;
}